<?php
return [
    /**
     * Debug Level:
     *
     * Production Mode:
     * false: No error messages, errors, or warnings shown.
     *
     * Development Mode:
     * true: Errors and warnings shown.
     */
    'debug' => filter_var(env('DEBUG', true), FILTER_VALIDATE_BOOLEAN),

    /**
     * Configure basic information about the application.
     *
     * - namespace - The namespace to find app classes under.
     * - defaultLocale - The default locale for translation, formatting currencies and numbers, date and time.
     * - encoding - The encoding used for HTML + database connections.
     * - base - The base directory the app resides in. If false this
     *   will be auto detected.
     * - dir - Name of app directory.
     * - webroot - The webroot directory.
     * - wwwRoot - The file path to webroot.
     * - baseUrl - To configure CakePHP to *not* use mod_rewrite and to
     *   use CakePHP pretty URLs, remove these .htaccess
     *   files:
     *      /.htaccess
     *      /webroot/.htaccess
     *   And uncomment the baseUrl key below.
     * - fullBaseUrl - A base URL to use for absolute links. When set to false (default)
     *   CakePHP generates required value based on `HTTP_HOST` environment variable.
     *   However, you can define it manually to optimize performance or if you
     *   are concerned about people manipulating the `Host` header.
     * - imageBaseUrl - Web path to the public images directory under webroot.
     * - cssBaseUrl - Web path to the public css directory under webroot.
     * - jsBaseUrl - Web path to the public js directory under webroot.
     * - paths - Configure paths for non class based resources. Supports the
     *   `plugins`, `templates`, `locales` subkeys, which allow the definition of
     *   paths for plugins, view templates and locale files respectively.
     */
    'App' => [
        'namespace' => 'App',
        'encoding' => env('APP_ENCODING', 'UTF-8'),
        'defaultLocale' => env('APP_DEFAULT_LOCALE', 'en_US'),
        'defaultTimezone' => env('APP_DEFAULT_TIMEZONE', 'UTC'),
        'base' => false,
        'dir' => 'src',
        'webroot' => 'webroot',
        'wwwRoot' => WWW_ROOT,
        //'baseUrl' => env('SCRIPT_NAME'),
        'fullBaseUrl' => false,
        'imageBaseUrl' => 'img/',
        'cssBaseUrl' => 'css/',
        'jsBaseUrl' => 'js/',
        'paths' => [
            'plugins' => [ROOT . DS . 'plugins' . DS],
            'templates' => [APP . 'Template' . DS],
            'locales' => [APP . 'Locale' . DS],
        ],
    ],

    /**
     * Security and encryption configuration
     *
     * - salt - A random string used in security hashing methods.
     *   The salt value is also used as the encryption key.
     *   You should treat it as extremely sensitive data.
     */
    'Security' => [
        'salt' => env('SECURITY_SALT', '05acabdea370e3a9271574b31dee3d49cf33053e1d533c393f36fedbc11ad55f'),
    ],

    /**
     * Apply timestamps with the last modified time to static assets (js, css, images).
     * Will append a querystring parameter containing the time the file was modified.
     * This is useful for busting browser caches.
     *
     * Set to true to apply timestamps when debug is true. Set to 'force' to always
     * enable timestamping regardless of debug value.
     */
    'Asset' => [
        //'timestamp' => true,
        // 'cacheTime' => '+1 year'
    ],

    /**
     * Configure the cache adapters.
     */
    'Cache' => [
        'default' => [
            'className' => 'Cake\Cache\Engine\FileEngine',
            'path' => CACHE,
            'url' => env('CACHE_DEFAULT_URL', null),
        ],

        /**
         * Configure the cache used for general framework caching.
         * Translation cache files are stored with this configuration.
         * Duration will be set to '+2 minutes' in bootstrap.php when debug = true
         * If you set 'className' => 'Null' core cache will be disabled.
         */
        '_cake_core_' => [
            'className' => 'Cake\Cache\Engine\FileEngine',
            'prefix' => 'myapp_cake_core_',
            'path' => CACHE . 'persistent/',
            'serialize' => true,
            'duration' => '+1 years',
            'url' => env('CACHE_CAKECORE_URL', null),
        ],

        /**
         * Configure the cache for model and datasource caches. This cache
         * configuration is used to store schema descriptions, and table listings
         * in connections.
         * Duration will be set to '+2 minutes' in bootstrap.php when debug = true
         */
        '_cake_model_' => [
            'className' => 'Cake\Cache\Engine\FileEngine',
            'prefix' => 'myapp_cake_model_',
            'path' => CACHE . 'models/',
            'serialize' => true,
            'duration' => '+1 years',
            'url' => env('CACHE_CAKEMODEL_URL', null),
        ],

        /**
         * Configure the cache for routes. The cached routes collection is built the
         * first time the routes are processed via `config/routes.php`.
         * Duration will be set to '+2 seconds' in bootstrap.php when debug = true
         */
        '_cake_routes_' => [
            'className' => 'Cake\Cache\Engine\FileEngine',
            'prefix' => 'myapp_cake_routes_',
            'path' => CACHE,
            'serialize' => true,
            'duration' => '+1 years',
            'url' => env('CACHE_CAKEROUTES_URL', null),
        ],
    ],

    /**
     * Configure the Error and Exception handlers used by your application.
     *
     * By default errors are displayed using Debugger, when debug is true and logged
     * by Cake\Log\Log when debug is false.
     *
     * In CLI environments exceptions will be printed to stderr with a backtrace.
     * In web environments an HTML page will be displayed for the exception.
     * With debug true, framework errors like Missing Controller will be displayed.
     * When debug is false, framework errors will be coerced into generic HTTP errors.
     *
     * Options:
     *
     * - `errorLevel` - int - The level of errors you are interested in capturing.
     * - `trace` - boolean - Whether or not backtraces should be included in
     *   logged errors/exceptions.
     * - `log` - boolean - Whether or not you want exceptions logged.
     * - `exceptionRenderer` - string - The class responsible for rendering
     *   uncaught exceptions. If you choose a custom class you should place
     *   the file for that class in src/Error. This class needs to implement a
     *   render method.
     * - `skipLog` - array - List of exceptions to skip for logging. Exceptions that
     *   extend one of the listed exceptions will also be skipped for logging.
     *   E.g.:
     *   `'skipLog' => ['Cake\Http\Exception\NotFoundException', 'Cake\Http\Exception\UnauthorizedException']`
     * - `extraFatalErrorMemory` - int - The number of megabytes to increase
     *   the memory limit by when a fatal error is encountered. This allows
     *   breathing room to complete logging or error handling.
     */
    'Error' => [
        'errorLevel' => E_ALL ^ E_USER_DEPRECATED,
        'exceptionRenderer' => 'Cake\Error\ExceptionRenderer',
        'skipLog' => [],
        'log' => true,
        'trace' => true,
    ],

    /**
     * Email configuration.
     *
     * By defining transports separately from delivery profiles you can easily
     * re-use transport configuration across multiple profiles.
     *
     * You can specify multiple configurations for production, development and
     * testing.
     *
     * Each transport needs a `className`. Valid options are as follows:
     *
     *  Mail   - Send using PHP mail function
     *  Smtp   - Send using SMTP
     *  Debug  - Do not send the email, just return the result
     *
     * You can add custom transports (or override existing transports) by adding the
     * appropriate file to src/Mailer/Transport. Transports should be named
     * 'YourTransport.php', where 'Your' is the name of the transport.
     */
    'EmailTransport' => [
        'default' => [
            'className' => 'Smtp',
            /*
             * The following keys are used in SMTP transports:
             */
            'host' => 'ssl://smtp.gmail.com',
            'port' => 465,
            'timeout' => 30,
            'username' => 'angopensource@gmail.com',
            'password' => 'Test@123',
            'client' => null,
            'tls' => null,
            'url' => env('EMAIL_TRANSPORT_DEFAULT_URL', null),
        ],
    ],

    /**
     * Email delivery profiles
     *
     * Delivery profiles allow you to predefine various properties about email
     * messages from your application and give the settings a name. This saves
     * duplication across your application and makes maintenance and development
     * easier. Each profile accepts a number of keys. See `Cake\Mailer\Email`
     * for more information.
     */
    'Email' => [
        'default' => [
            'transport' => 'default',
            'from' => 'you@localhost',
            //'charset' => 'utf-8',
            //'headerCharset' => 'utf-8',
        ],
    ],

    /**
     * Connection information used by the ORM to connect
     * to your application's datastores.
     *
     * ### Notes
     * - Drivers include Mysql Postgres Sqlite Sqlserver
     *   See vendor\cakephp\cakephp\src\Database\Driver for complete list
     * - Do not use periods in database name - it may lead to error.
     *   See https://github.com/cakephp/cakephp/issues/6471 for details.
     * - 'encoding' is recommended to be set to full UTF-8 4-Byte support.
     *   E.g set it to 'utf8mb4' in MariaDB and MySQL and 'utf8' for any
     *   other RDBMS.
     */
    'Datasources' => [
        'default' => [
            'className' => 'Cake\Database\Connection',
            'driver' => 'Cake\Database\Driver\Mysql',
            'persistent' => false,
            'host' => 'localhost',
            /*
             * CakePHP will use the default DB port based on the driver selected
             * MySQL on MAMP uses port 8889, MAMP users will want to uncomment
             * the following line and set the port accordingly
             */
            //'port' => 'non_standard_port_number',
            'username' => 'smwconnect',
            'password' => 'mYE!6i6*g8',
            'database' => 'super_machines_staging',
            /*
             * You do not need to set this flag to use full utf-8 encoding (internal default since CakePHP 3.6).
             */
            //'encoding' => 'utf8mb4',
            'timezone' => 'UTC',
            'flags' => [],
            'cacheMetadata' => true,
            'log' => false,

            /**
             * Set identifier quoting to true if you are using reserved words or
             * special characters in your table or column names. Enabling this
             * setting will result in queries built using the Query Builder having
             * identifiers quoted when creating SQL. It should be noted that this
             * decreases performance because each query needs to be traversed and
             * manipulated before being executed.
             */
            'quoteIdentifiers' => false,

            /**
             * During development, if using MySQL < 5.6, uncommenting the
             * following line could boost the speed at which schema metadata is
             * fetched from the database. It can also be set directly with the
             * mysql configuration directive 'innodb_stats_on_metadata = 0'
             * which is the recommended value in production environments
             */
            //'init' => ['SET GLOBAL innodb_stats_on_metadata = 0'],

            'url' => env('DATABASE_URL', null),
        ],

        /**
         * The test connection is used during the test suite.
         */
        'test' => [
            'className' => 'Cake\Database\Connection',
            'driver' => 'Cake\Database\Driver\Mysql',
            'persistent' => false,
            'host' => 'localhost',
            //'port' => 'non_standard_port_number',
            'username' => 'my_app',
            'password' => 'secret',
            'database' => 'test_myapp',
            //'encoding' => 'utf8mb4',
            'timezone' => 'UTC',
            'cacheMetadata' => true,
            'quoteIdentifiers' => false,
            'log' => false,
            //'init' => ['SET GLOBAL innodb_stats_on_metadata = 0'],
            'url' => env('DATABASE_TEST_URL', null),
        ],
    ],

    /**
     * Configures logging options
     */
    'Log' => [
        'debug' => [
            'className' => 'Cake\Log\Engine\FileLog',
            'path' => LOGS,
            'file' => 'debug',
            'url' => env('LOG_DEBUG_URL', null),
            'scopes' => false,
            'levels' => ['notice', 'info', 'debug'],
        ],
        'error' => [
            'className' => 'Cake\Log\Engine\FileLog',
            'path' => LOGS,
            'file' => 'error',
            'url' => env('LOG_ERROR_URL', null),
            'scopes' => false,
            'levels' => ['warning', 'error', 'critical', 'alert', 'emergency'],
        ],
        // To enable this dedicated query log, you need set your datasource's log flag to true
        'queries' => [
            'className' => 'Cake\Log\Engine\FileLog',
            'path' => LOGS,
            'file' => 'queries',
            'url' => env('LOG_QUERIES_URL', null),
            'scopes' => ['queriesLog'],
        ],
    ],

    /**
     * Session configuration.
     *
     * Contains an array of settings to use for session configuration. The
     * `defaults` key is used to define a default preset to use for sessions, any
     * settings declared here will override the settings of the default config.
     *
     * ## Options
     *
     * - `cookie` - The name of the cookie to use. Defaults to 'CAKEPHP'. Avoid using `.` in cookie names,
     *   as PHP will drop sessions from cookies with `.` in the name.
     * - `cookiePath` - The url path for which session cookie is set. Maps to the
     *   `session.cookie_path` php.ini config. Defaults to base path of app.
     * - `timeout` - The time in minutes the session should be valid for.
     *    Pass 0 to disable checking timeout.
     *    Please note that php.ini's session.gc_maxlifetime must be equal to or greater
     *    than the largest Session['timeout'] in all served websites for it to have the
     *    desired effect.
     * - `defaults` - The default configuration set to use as a basis for your session.
     *    There are four built-in options: php, cake, cache, database.
     * - `handler` - Can be used to enable a custom session handler. Expects an
     *    array with at least the `engine` key, being the name of the Session engine
     *    class to use for managing the session. CakePHP bundles the `CacheSession`
     *    and `DatabaseSession` engines.
     * - `ini` - An associative array of additional ini values to set.
     *
     * The built-in `defaults` options are:
     *
     * - 'php' - Uses settings defined in your php.ini.
     * - 'cake' - Saves session files in CakePHP's /tmp directory.
     * - 'database' - Uses CakePHP's database sessions.
     * - 'cache' - Use the Cache class to save sessions.
     *
     * To define a custom session handler, save it at src/Network/Session/<name>.php.
     * Make sure the class implements PHP's `SessionHandlerInterface` and set
     * Session.handler to <name>
     *
     * To use database sessions, load the SQL file located at config/schema/sessions.sql
     */
    'Session' => [
        'defaults' => 'php',
		'timeout'	=> 24*60,
    ],
    #active status
    'activeStatus' => [
        0 => 'In Active',
        1 => 'Active',
    ],
    /*
	 * user default password on reset
	*/
	'defaultPassword' => '19Smw2019(+',
    /*
	 * import button hide/show
	*/
	'ImportHide' => false,
	/*
	 * audit log
	*/
	'AuditStash' => [
		'persister' => 'AuditStash\Persister\TablePersister'
	],
	/*
	* year plans
	*/
	'FinancialYears' => [
		'2019-2020' => '2019-2020',
		'2020-2021' => '2020-2021',
		'2021-2022' => '2021-2022',
		'2022-2023' => '2022-2023',
		'2023-2024' => '2023-2024',
		'2024-2025' => '2024-2025'
	],
	/*
	 *financial month starts and end
	*/
	'FinancialMonths' => [
		'start'	=> 'March',
		'end'	=> 'April'
	],
    /*
    * quarter months
    */
    'QuarterMonths' => [
		'Quarter 1'	=> ['April', 'May', 'June'],
		'Quarter 2'	=> ['July', 'August', 'September'],
		'Quarter 3'	=> ['October', 'November', 'December'],
		'Quarter 4'	=> ['January', 'February', 'March'],
	],
	/*
	* quater duration
	*/
	'QuaterDuration' => 4,
     /*
     * tracksheet assembly status
    */
    'TrackSheetProductStatus' => [
        0 => [
            'status'    => 'Pending',
            'span-class'=> 'label label-default',
            'th-class'  => '',
        ],
        1 => [
            'status'    => 'Completed',
            'span-class'=> 'label label-green',
            'th-class'  => 'bg-green',
        ],
        2 => [
            'status'    => 'Delay',
            'span-class'=> 'label label-warning',
            'th-class'  => 'bg-yellow',
        ],
        3 => [
            'status'    => 'Rework',
            'span-class'=> 'label label-danger',
            'th-class'  => 'bg-red',
        ],
        4 => [
            'status'    => 'Dispacthed',
            'span-class'=> 'label label-primary',
            'th-class'  => 'bg-red',
        ],
    ],
    /*
     * tracksheet assembly status
    */
    'TrackSheetAssemblyStatus' => [
        0 => [
                # blank nothing happen
                'tick'      => 'status/20/blank.png',
                'ts-class'  => '',
             ],
        1 => [
                # component ready in all assembly
                'tick'      => 'status/24/assembly-full-completed.png',
                'ts-class'  => 'subassembly-qc-accepted',
             ],
        2 => [
                # component waiting any one assembly
                'tick'      => 'status/24/assembly-full-delay.png',
                'ts-class'  => 'subassembly-to-qc-moved',
             ],
        3 => [
                # component Rework any one assembly
                'tick'      => 'status/24/assembly-full-reject.png',
                'ts-class'  => 'subassembly-qc-rejection',
             ],
        4 => [
                # component Rework any one assembly
                'tick'      => 'status/20/blank.png',
                'ts-class'  => '',
             ],
    ],
    /*
     * tracksheet component status
    */
    'TrackSheetComponentStatus' => [
        0 => [
            'status' => 'Blank',
            # blank nothing happen
            1   => ['class' => '', 'tick' => 'status/20/blank.png'],
            2   => ['class' => '', 'tick' => 'status/20/blank.png'],
            3   => ['class' => '', 'tick' => 'status/20/blank.png'],
            4   => ['class' => 'do-select supervisor-tick', 'tick' => 'status/20/blank.png'],
            5   => ['class' => '', 'tick' => 'status/20/blank.png'],
            6   => ['class' => '', 'tick' => 'status/20/blank.png'],
            7   => ['class' => '', 'tick' => 'status/20/blank.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        1 => [
            'status' => 'In Progress',
            # component in progress
            1   => ['class' => '', 'tick' => 'status/20/supervisor-progress.png'],
            2   => ['class' => '', 'tick' => 'status/20/supervisor-progress.png'],
            3   => ['class' => '', 'tick' => 'status/20/supervisor-progress.png'],
            4   => ['class' => 'do-select supervisor-progress-undo', 'tick' => 'status/20/supervisor-progress.png'],
            5   => ['class' => 'supervisor-progress-done', 'tick' => 'status/20/supervisor-progress.png'],
            6   => ['class' => '', 'tick' => 'status/20/blank.png'],
            7   => ['class' => '', 'tick' => 'status/20/blank.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        2 => [
            'status' => 'Completed',
            # component ready
            1   => ['class' => '', 'tick' => 'status/20/supervisor-completed.png'],
            2   => ['class' => '', 'tick' => 'status/20/supervisor-completed.png'],
            3   => ['class' => '', 'tick' => 'status/20/supervisor-completed.png'],
            4   => ['class' => 'supervisor-completed-undo', 'tick' => 'status/20/supervisor-completed.png'],
            5   => ['class' => 'do-select qc-done', 'tick' => 'status/20/supervisor-completed.png'],
            6   => ['class' => '', 'tick' => 'status/20/blank.png'],
            7   => ['class' => '', 'tick' => 'status/20/blank.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        3 => [
            'status' => 'QC Waiting Level 1',
            # QC delay first level (yellow background)
            1   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-delay.png'],
            2   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-delay.png'],
            3   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-delay.png'],
            4   => ['class' => 'bg-yellow-active supervisor-completed-undo', 'tick' => 'status/20/qc-delay.png'],
            5   => ['class' => 'bg-yellow-active do-select qc-done', 'tick' => 'status/20/qc-delay.png'],
            6   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-delay-assembly.png'],
            7   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-delay-assembly.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        4 => [
            'status' => 'QC Waiting Level 2',
            # QC delay second level (red background)
            1   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-delay.png'],
            2   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-delay.png'],
            3   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-delay.png'],
            4   => ['class' => 'bg-red-active supervisor-completed-undo', 'tick' => 'status/20/qc-delay.png'],
            5   => ['class' => 'bg-red-active do-select qc-done', 'tick' => 'status/20/qc-delay.png'],
            6   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-delay-assembly.png'],
            7   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-delay-assembly.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        5 => [
            'status' => 'QC Partial Rework',
            # QC Partial Rework cross tick with yellow background
            1   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-reject.png'],
            2   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-reject.png'],
            3   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-reject.png'],
            4   => ['class' => 'bg-yellow-active supervisor-re-tick', 'tick' => 'status/20/qc-reject.png'],
            5   => ['class' => 'bg-yellow-active qc-redo', 'tick' => 'status/20/qc-reject.png'],
            6   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-reject.png'],
            7   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-reject.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        6 => [
            'status' => 'QC Rework',
            # QC Rework cross tick with red background
            1   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-reject.png'],
            2   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-reject.png'],
            3   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-reject.png'],
            4   => ['class' => 'bg-red-active supervisor-re-tick', 'tick' => 'status/20/qc-reject.png'],
            5   => ['class' => 'bg-red-active qc-redo', 'tick' => 'status/20/qc-reject.png'],
            6   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-reject.png'],
            7   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-reject.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        7 => [
            'status' => 'QC Conditional Completed',
            # QC conditional completed
            1   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
            2   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
            3   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
            4   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
            5   => ['class' => 'qc-undo', 'tick' => 'status/20/qc-complete.png'],
            6   => ['class' => 'do-select assembly-done', 'tick' => 'status/20/qc-complete.png'],
            7   => ['class' => 'do-select assembly-done', 'tick' => 'status/20/qc-complete.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        8 => [
            'status' => 'QC Completed',
            # QC completed
            1   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
            2   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
            3   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
            4   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
            5   => ['class' => 'qc-undo', 'tick' => 'status/20/qc-complete.png'],
            6   => ['class' => 'do-select assembly-done', 'tick' => 'status/20/qc-complete.png'],
            7   => ['class' => 'do-select assembly-done', 'tick' => 'status/20/qc-complete.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        9 => [
            'status' => 'Assembly Waiting Level 1',
            # QC completed delay in assembly first level (yellow background)
            1   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/assembly-delay.png'],
            2   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/assembly-delay.png'],
            3   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/assembly-delay.png'],
            4   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/assembly-delay.png'],
            5   => ['class' => 'bg-yellow-active qc-undo', 'tick' => 'status/20/assembly-delay.png'],
            6   => ['class' => 'bg-yellow-active do-select assembly-done', 'tick' => 'status/20/assembly-delay.png'],
            7   => ['class' => 'bg-yellow-active do-select assembly-done', 'tick' => 'status/20/assembly-delay.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        10 => [
            'status' => 'Assembly Waiting Level 2',
            # QC completed delay in assembly second level (red background)
            1   => ['class' => 'bg-red-active ', 'tick' => 'status/20/assembly-delay.png'],
            2   => ['class' => 'bg-red-active ', 'tick' => 'status/20/assembly-delay.png'],
            3   => ['class' => 'bg-red-active ', 'tick' => 'status/20/assembly-delay.png'],
            4   => ['class' => 'bg-red-active ', 'tick' => 'status/20/assembly-delay.png'],
            5   => ['class' => 'bg-red-active qc-undo', 'tick' => 'status/20/assembly-delay.png'],
            6   => ['class' => 'bg-red-active do-select assembly-done', 'tick' => 'status/20/assembly-delay.png'],
            7   => ['class' => 'bg-red-active do-select assembly-done', 'tick' => 'status/20/assembly-delay.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        11 => [
            'status' => 'Assembly Partial Rework',
            # Assembly Partial Rework cross tick with yellow background
            1   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/assembly-reject.png'],
            2   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/assembly-reject.png'],
            3   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/assembly-reject.png'],
            4   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/assembly-reject.png'],
            5   => ['class' => 'bg-yellow-active qc-re-done', 'tick' => 'status/20/assembly-reject.png'],
            6   => ['class' => 'bg-yellow-active assembly-redo', 'tick' => 'status/20/assembly-reject.png'],
            7   => ['class' => 'bg-yellow-active assembly-redo', 'tick' => 'status/20/assembly-reject.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        12 => [
            'status' => 'Assembly Rework',
            # assembly Rework
            1   => ['class' => 'bg-red-active ', 'tick' => 'status/20/assembly-reject.png'],
            2   => ['class' => 'bg-red-active ', 'tick' => 'status/20/assembly-reject.png'],
            3   => ['class' => 'bg-red-active ', 'tick' => 'status/20/assembly-reject.png'],
            4   => ['class' => 'bg-red-active ', 'tick' => 'status/20/assembly-reject.png'],
            5   => ['class' => 'bg-red-active qc-re-done', 'tick' => 'status/20/assembly-reject.png'],
            6   => ['class' => 'bg-red-active assembly-redo', 'tick' => 'status/20/assembly-reject.png'],
            7   => ['class' => 'bg-red-active assembly-redo', 'tick' => 'status/20/assembly-reject.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        13 => [
            'status' => 'Assembly Conditional Completed',
            # assembly conditional completed
            1   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
            2   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
            3   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
            4   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
            5   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
            6   => ['class' => 'assembly-undo', 'tick' => 'status/20/assembly-accept.png'],
            7   => ['class' => 'assembly-undo', 'tick' => 'status/20/assembly-accept.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        14 => [
            'status' => 'Assembly Completed',
            # assembly completed
            1   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
            2   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
            3   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
            4   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
            5   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
            6   => ['class' => 'assembly-undo', 'tick' => 'status/20/assembly-accept.png'],
            7   => ['class' => 'assembly-undo', 'tick' => 'status/20/assembly-accept.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        15 => [
            'status' => 'Group Production Delay Level 1',
            # blank nothing happen
            1   => ['class' => '', 'tick' => 'status/20/group-delay-1.png'],
            2   => ['class' => '', 'tick' => 'status/20/group-delay-1.png'],
            3   => ['class' => '', 'tick' => 'status/20/group-delay-1.png'],
            4   => ['class' => 'do-select supervisor-tick', 'tick' => 'status/20/group-delay-1.png'],
            5   => ['class' => '', 'tick' => 'status/20/group-delay-1.png'],
            6   => ['class' => '', 'tick' => 'status/20/group-delay-1.png'],
            7   => ['class' => '', 'tick' => 'status/20/group-delay-1.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
        16 => [
            'status' => 'Group Production Delay Level 2',
            # blank nothing happen
            1   => ['class' => '', 'tick' => 'status/20/group-delay-2.png'],
            2   => ['class' => '', 'tick' => 'status/20/group-delay-2.png'],
            3   => ['class' => '', 'tick' => 'status/20/group-delay-2.png'],
            4   => ['class' => 'do-select supervisor-tick', 'tick' => 'status/20/group-delay-2.png'],
            5   => ['class' => '', 'tick' => 'status/20/group-delay-2.png'],
            6   => ['class' => '', 'tick' => 'status/20/group-delay-2.png'],
            7   => ['class' => '', 'tick' => 'status/20/group-delay-2.png'],
            8   => ['class' => '', 'tick' => 'status/20/blank.png'],
        ],
    ],
#spare tracking
    'TrackSheetSpareStatus' => [
        0 => [
                'status' => 'Blank',
                # blank nothing happen
                1   => ['class' => '', 'tick' => 'status/20/blank.png'],
                2   => ['class' => '', 'tick' => 'status/20/blank.png'],
                3   => ['class' => '', 'tick' => 'status/20/blank.png'],
                4   => ['class' => 'supervisor-spare-tick', 'tick' => 'status/20/blank.png'],
                5   => ['class' => '', 'tick' => 'status/20/blank.png'],
                6   => ['class' => '', 'tick' => 'status/20/blank.png'],
                7   => ['class' => '', 'tick' => 'status/20/blank.png'],
                8   => ['class' => '', 'tick' => 'status/20/blank.png'],
             ],
        1 => [
                'status' => 'In Progress',
                # component in progress
                1   => ['class' => '', 'tick' => 'status/20/supervisor-progress.png'],
                2   => ['class' => '', 'tick' => 'status/20/supervisor-progress.png'],
                3   => ['class' => '', 'tick' => 'status/20/supervisor-progress.png'],
                4   => ['class' => 'supervisor-spare-progress-undo', 'tick' => 'status/20/supervisor-progress.png'],
                5   => ['class' => 'supervisor-spare-progress-done', 'tick' => 'status/20/supervisor-progress.png'],
                6   => ['class' => '', 'tick' => 'status/20/blank.png'],
                7   => ['class' => '', 'tick' => 'status/20/blank.png'],
                8   => ['class' => '', 'tick' => 'status/20/blank.png'],
             ],
        2 => [
                'status' => 'Completed',
                # component ready
                1   => ['class' => '', 'tick' => 'status/20/supervisor-completed.png'],
                2   => ['class' => '', 'tick' => 'status/20/supervisor-completed.png'],
                3   => ['class' => '', 'tick' => 'status/20/supervisor-completed.png'],
                4   => ['class' => 'supervisor-spare-completed-undo', 'tick' => 'status/20/supervisor-completed.png'],
                5   => ['class' => 'qc-spare-done', 'tick' => 'status/20/supervisor-completed.png'],
                6   => ['class' => '', 'tick' => 'status/20/blank.png'],
                7   => ['class' => '', 'tick' => 'status/20/blank.png'],
                8   => ['class' => '', 'tick' => 'status/20/blank.png'],
             ],
        3 => [
                'status' => 'QC Waiting Level 1',
                # QC delay first level (yellow background)
                1   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-delay.png'],
                2   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-delay.png'],
                3   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-delay.png'],
                4   => ['class' => 'bg-yellow-active supervisor-completed-undo', 'tick' => 'status/20/qc-delay.png'],
                5   => ['class' => 'bg-yellow-active qc-done', 'tick' => 'status/20/qc-delay.png'],
                6   => ['class' => '', 'tick' => 'status/20/qc-delay-assembly.png'],
                7   => ['class' => '', 'tick' => 'status/20/qc-delay-assembly.png'],
                8   => ['class' => '', 'tick' => 'status/20/blank.png'],
             ],
        4 => [
                'status' => 'QC Waiting Level 2',
                # QC delay second level (red background)
                1   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-delay.png'],
                2   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-delay.png'],
                3   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-delay.png'],
                4   => ['class' => 'bg-red-active supervisor-completed-undo', 'tick' => 'status/20/qc-delay.png'],
                5   => ['class' => 'bg-red-active qc-spare-done', 'tick' => 'status/20/qc-delay.png'],
                6   => ['class' => '', 'tick' => 'status/20/blank.png'],
                7   => ['class' => '', 'tick' => 'status/20/blank.png'],
                8   => ['class' => '', 'tick' => 'status/20/blank.png'],
             ],
        5 => [
                'status' => 'QC Partial Rework',
                # QC Partial Rework cross tick with yellow background
                1   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-reject.png'],
                2   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-reject.png'],
                3   => ['class' => 'bg-yellow-active ', 'tick' => 'status/20/qc-reject.png'],
                4   => ['class' => 'bg-yellow-active supervisor-spare-re-tick', 'tick' => 'status/20/qc-reject.png'],
                5   => ['class' => 'bg-yellow-active qc-spare-redo', 'tick' => 'status/20/qc-reject.png'],
                6   => ['class' => '', 'tick' => 'status/20/blank.png'],
                7   => ['class' => '', 'tick' => 'status/20/blank.png'],
                8   => ['class' => '', 'tick' => 'status/20/blank.png'],
            ],
        6 => [
                'status' => 'QC Rework',
                # QC Rework cross tick with red background
                1   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-reject.png'],
                2   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-reject.png'],
                3   => ['class' => 'bg-red-active ', 'tick' => 'status/20/qc-reject.png'],
                4   => ['class' => 'bg-red-active supervisor-spare-re-tick', 'tick' => 'status/20/qc-reject.png'],
                5   => ['class' => 'bg-red-active qc-spare-redo', 'tick' => 'status/20/qc-reject.png'],
                6   => ['class' => '', 'tick' => 'status/20/blank.png'],
                7   => ['class' => '', 'tick' => 'status/20/blank.png'],
                8   => ['class' => '', 'tick' => 'status/20/blank.png'],
             ],
        7 => [
                'status' => 'QC Conditional Completed',
                # QC conditional completed
                1   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
                2   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
                3   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
                4   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
                5   => ['class' => 'qc-spare-undo', 'tick' => 'status/20/qc-complete.png'],
                6   => ['class' => 'store-done', 'tick' => 'status/20/blank.png'],
                7   => ['class' => 'store-done', 'tick' => 'status/20/blank.png'],
                8   => ['class' => '', 'tick' => 'status/20/blank.png'],
             ],
        8 => [
                'status' => 'QC Completed',
                # QC completed
                1   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
                2   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
                3   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
                4   => ['class' => '', 'tick' => 'status/20/qc-complete.png'],
                5   => ['class' => 'qc-spare-undo', 'tick' => 'status/20/qc-complete.png'],
                6   => ['class' => '', 'tick' => 'status/20/blank.png'],
                7   => ['class' => '', 'tick' => 'status/20/blank.png'],
                8   => ['class' => '', 'tick' => 'status/20/blank.png'],
             ],
        9 => [
                'status' => 'Store Accepted',
                # store user accepted
                1   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
                2   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
                3   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
                4   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
                5   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
                6   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
                7   => ['class' => '', 'tick' => 'status/20/assembly-accept.png'],
                8   => ['class' => '', 'tick' => 'status/20/blank.png'],
             ],
    ],
#component tags
    'tag' => [
        0 => 'In',
        1 => 'Out',
        2 => 'In/Out',
    ],
#inter group status
    'intergroupStatus' => [
        0 => 'New',
        1 => 'Acknowledge',
        2 => 'Transfered',
        3 => 'Verified',
    ],
    'purchase_indent'           => [
        0 => 'New',
        1 => 'Forwarded',
        2 => 'Adhoc-Waiting for approval',
        3 => 'Adhoc Rejected',
        4 => 'Raised Purchase Order',
        5 => 'Waiting for approval',
        6 => 'In Progress',
        7 => 'Delivered',
        8 => 'Hold',
        9 => 'Rejected',
    ],
    'job_order'                 => [
        0 => 'Waiting for approval',
        1 => 'Forwarded',
        2 => 'Raised Purchase Order',
        3 => 'In Progress',
        4 => 'Delivered',
        5 => 'Hold',
        6 => 'Rejected',
    ],
    'store_order'               => [
        0 => 'Waiting for approval',
        1 => 'Forwarded',
        2 => 'Raised Purchase Order',
        3 => 'In Progress',
        4 => 'Delivered',
        5 => 'Hold',
        6 => 'Rejected',
    ],
    'purchase_order'            => [
        1 => 'Waiting for Approval',
        2 => 'In Progress',
        3 => 'Delivered',
        4 => 'Hold',
        5 => 'Rejected',
    ],
    'purchase_order_terms'      => [
        'Delivered at Place'    => 'Delivered at Place',
        'Delivered at Terminal' => 'Delivered at Terminal',
        'Delivered At Frontier' => 'Delivered At Frontier',
        'Free On Board'         => 'Free On Board',
        'Free Carrier'          => 'Free Carrier',
    ],
#cakePdf initiate
    'CakePdf' => [
        'engine' => [
            'className' => 'CakePdf.WkHtmlToPdf',
            'binary' => '/usr/local/bin/wkhtmltopdf',
            'options' => [
                'print-media-type' => false,
                'outline' => true,
                'dpi' => 96
            ],
        ]
    ],
#machine running days
    'machineRunningDays'    => 25,
    'machineRunningMonths'  => 12,
#component drawing image config
    'DrawingPath' => 'http://192.168.1.7/smw/',
#group delay based on product
    'GroupDelay' => 2,
    'QuarterWorkingDays' => 75,

#default allowed methods
    'userDefaultPrivilleges' => [
    #user role
        'userPrivillege',
    #users
        'changePassword',
        'profileView',
        'profileEdit',
        'resetPassword',
        'login',
        'logout',
    #anual plan
        'freezCapacityPlan',
        'capacityPlanView',
        'prepareQuarterPlan',
    #quarter plan
        'eventualProducts',
        'eventualComponentMapping',
        'mapEventualComponents',
        'freezeEventualProducts',
    ],
];
