<?php
namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

use Cake\Network\Session;
use Cake\Routing\Router;

/**
 * BuyerLocations Model
 *
 * @property \App\Model\Table\ForecastMaterialsTable|\Cake\ORM\Association\HasMany $ForecastMaterials
 * @property \App\Model\Table\InvoicesTable|\Cake\ORM\Association\HasMany $Invoices
 * @property \App\Model\Table\PurchaseOrdersTable|\Cake\ORM\Association\HasMany $PurchaseOrders
 * @property \App\Model\Table\BroadcastsTable|\Cake\ORM\Association\BelongsToMany $Broadcasts
 * @property \App\Model\Table\BuyerUsersTable|\Cake\ORM\Association\BelongsToMany $BuyerUsers
 *
 * @method \App\Model\Entity\BuyerLocation get($primaryKey, $options = [])
 * @method \App\Model\Entity\BuyerLocation newEntity($data = null, array $options = [])
 * @method \App\Model\Entity\BuyerLocation[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\BuyerLocation|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\BuyerLocation patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\BuyerLocation[] patchEntities($entities, array $data, array $options = [])
 * @method \App\Model\Entity\BuyerLocation findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class BuyerLocationsTable extends Table
{

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config)
    {
        parent::initialize($config);

        $this->setTable('buyer_locations');
        $this->setDisplayField('display_name');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->hasMany('ForecastMaterials', [
            'foreignKey' => 'buyer_location_id'
        ]);
        $this->hasMany('IndependentSchedules', [
            'foreignKey' => 'buyer_location_id'
        ]);
        $this->hasMany('Invoices', [
            'foreignKey' => 'buyer_location_id'
        ]);
        $this->hasMany('PurchaseOrders', [
            'foreignKey' => 'buyer_location_id'
        ]);
        $this->belongsToMany('Broadcasts', [
            'foreignKey' => 'buyer_location_id',
            'targetForeignKey' => 'broadcast_id',
            'joinTable' => 'buyer_locations_broadcasts'
        ]);
        $this->belongsToMany('BuyerUsers', [
            'foreignKey' => 'buyer_location_id',
            'targetForeignKey' => 'buyer_user_id',
            'joinTable' => 'buyer_users_buyer_locations'
        ]);
        $this->belongsToMany('Tickets', [
            'foreignKey' => 'buyer_location_id',
            'targetForeignKey' => 'ticket_id',
            'joinTable' => 'tickets_buyer_locations'
        ]);       

        // Add the behaviour to your table
        $this->addBehavior('Search.Search');
    }


    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmpty('id', 'create');

        $validator
            ->scalar('code')
            ->maxLength('code', 12)
            ->requirePresence('code', 'create')
            ->notEmpty('code')
            ->add('code', 'unique', ['rule' => 'validateUnique', 'provider' => 'table']);

        $validator
            ->scalar('name')
            ->maxLength('name', 75)
            ->requirePresence('name', 'create')
            ->notEmpty('name');

        $validator
            ->scalar('address')
            ->requirePresence('address', 'create')
            ->notEmpty('address');

        $validator
            ->email('email')
            ->allowEmpty('email');

        $validator
            ->scalar('phone')
            ->maxLength('phone', 50)
            ->allowEmpty('phone');

        $validator
            ->scalar('mobile')
            ->maxLength('mobile', 50)
            ->allowEmpty('mobile');

        $validator
            ->scalar('state_code')
            ->maxLength('state_code', 4)
            ->allowEmpty('state_code');

        $validator
            ->scalar('gstin')
            ->maxLength('gstin', 50)
            ->allowEmpty('gstin');

        $validator
            ->scalar('status')
            ->allowEmpty('status');

        return $validator;
    }

    /**
     * @return \Search\Manager
     */
    public function searchManager()
    {

        $searchManager = $this->behaviors()->Search->searchManager();
        $searchManager->value('status');
        
        if(isset(Router::getRequest()->query['st'])){
            if(Router::getRequest()->query['st'] != ''){
                $field = [Router::getRequest()->query['st']];
            }else {
                $field = ['code', 'name', 'email', 'phone', 'mobile'];
            }

            $searchManager->add('q', 'Search.Like', [
                'before' => true,
                'after' => true,
                'fieldMode' => 'OR',
                'comparison' => 'LIKE',
                'wildcardAny' => '*',
                'wildcardOne' => '?',
                'field' => $field
            ]);
        }
        
        return $searchManager;
    }

    #Apply Condition to Provide Records which belongs to the Logged in User
    public function beforeFind($event, $query, $options, $primary) { 
        $session = new Session();
        // pr($session->read('Auth.Buyer.materials_divisions'));

        if($session->read('Auth.Buyer.buyer_locations')) {
            $buyer_locations = array_map(function ($entry) {
                return $entry['id'];
            }, $session->read('Auth.Buyer.buyer_locations'));

            $prefix     = Router::getRequest()->params['prefix'];
            $controller = Router::getRequest()->params['controller'];
            $action     = Router::getRequest()->params['action'];

            if(
                $prefix == 'buyer' && 
                !($controller == 'forecast' && ($action != 'import' && $action != 'confirm'))  
            ){
                $query->where(['BuyerLocations.id IN ' => $buyer_locations]);
            }

            // pr($query); exit;

            return $query;
        }
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules)
    {
        $rules->add($rules->isUnique(['email']));
        $rules->add($rules->isUnique(['code']));

        return $rules;
    }
}
